//
//  VisilabsSideBar.swift
//  VisilabsIOS
//
//  Created by Orhun Akmil on 13.01.2022.
//

import Foundation
import UIKit


class RDDrawerViewController : RDBaseNotificationViewController {
    
    var position: CGPoint?
    var model = DrawerViewModel()
    var globDrawerView : drawerView?
    var drawerOpen:Bool = false
    var drawerFirstPosition:CGPoint?
    var titleLenght = 12
    var shouldDismissed = false
    
    
    init(model:DrawerServiceModel?) {
        super.init(nibName: nil, bundle: nil)
        self.model = RDDrawerViewControllerModel().mapServiceModelToNeededModel(serviceModel: model)
        let drawerView : drawerView = UIView.fromNib()
        drawerView.drawerModel = self.model
        globDrawerView = drawerView
        addTapGestureToDrawerMiniView()
        addTapGestureToImageOfGranDrawer()
        if self.model.isCircle {
            self.globDrawerView!.isHidden = true
            self.model.miniDrawerWidth = self.model.miniDrawerWidthForCircle / 2
        }
        self.view = drawerView
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        drawerFirstPosition = self.window?.layer.position
        if model.isCircle {
            configureCircleDrawer()
        }
        initializeData()
        
    }
    
    func initializeData() {
        
        if model.screenXcoordinate == .right {
            globDrawerView?.leftDrawerMiniContentImageView.image = model.miniDrawerContentImage
            globDrawerView?.leftTitleLabel.text = model.titleString
            globDrawerView?.leftTitleLabel.font = model.miniDrawerTextFont
            globDrawerView?.leftTitleLabel.textColor = model.miniDrawerTextColor
            globDrawerView?.leftDrawerMiniImageView.image = model.miniDrawerBackgroundImage
            globDrawerView?.leftDrawerMiniImageView.backgroundColor = model.miniDrawerBackgroundColor
            globDrawerView?.leftDrawerMiniArrow.textColor = model.arrowColor
        } else {
            globDrawerView?.rightDrawerMiniContentImageView.image = model.miniDrawerContentImage
            globDrawerView?.rightTitleLabel.text = model.titleString
            globDrawerView?.rightTitleLabel.font = model.miniDrawerTextFont
            globDrawerView?.rightTitleLabel.textColor = model.miniDrawerTextColor
            globDrawerView?.rightDrawerMiniImageView.image = model.miniDrawerBackgroundImage
            globDrawerView?.rightDrawerMiniImageView.backgroundColor = model.miniDrawerBackgroundColor
            globDrawerView?.rightDrawerMiniArrow.textColor = model.arrowColor
        }
        globDrawerView?.drawerGrandImageView.image = model.drawerBackgroundImage
        globDrawerView?.drawerGrandImageView.backgroundColor = model.drawerBackgroundColor
        globDrawerView?.drawerGrandContentImageView.image = model.drawerContentImage

    }
    
    func createDummyModel()  -> DrawerViewModel {
        var model = DrawerViewModel()
        
        model.titleString = "DenemeDeneme"
        model.isCircle = true
        model.screenYcoordinate = .middle
        model.screenXcoordinate = .right
        model.labelType = .downToUp

        return model
    }
        
    func configureCircleDrawer() {

        if model.screenXcoordinate == .right {
            
            self.globDrawerView?.leftDrawerMiniView.layer.zPosition = -1
            self.globDrawerView?.leftDrawerWidthConstraint.constant = self.model.miniDrawerWidthForCircle
            self.globDrawerView?.leftDrawerMiniView.layer.cornerRadius = self.model.miniDrawerWidthForCircle / 2
            self.globDrawerView?.leftDrawerTrailingConstraint.constant = -((self.globDrawerView?.leftDrawerWidthConstraint.constant)! / 2)
            self.globDrawerView?.leftDrawerTitleLabelCenterXConstraint.constant =  self.model.xCoordPaddingConstant
            self.globDrawerView?.leftDrawerContentImageCenterXConstraint.constant =  2
    
            if self.model.titleString.count > 0 {
                self.globDrawerView?.leftDrawerMiniContentImageView.isHidden = true
                self.globDrawerView?.leftDrawerMiniContentImageTopConstraint.constant *= 1.6
                self.globDrawerView?.leftDrawerMiniContentImageBottomConstraint.constant *= 3
                self.globDrawerView?.leftDrawerMiniContentImageLeadingConstraint.constant = 20
                self.globDrawerView?.leftDrawerMiniContentImageTrailingConstraint.constant += (self.globDrawerView?.drawerModel!.miniDrawerWidth)!
            } else {
                self.globDrawerView?.leftDrawerMiniContentImageTopConstraint.constant *=  3
                self.globDrawerView?.leftDrawerMiniContentImageBottomConstraint.constant *= 3
                self.globDrawerView?.leftDrawerMiniContentImageLeadingConstraint.constant = 20
                self.globDrawerView?.leftDrawerMiniContentImageTrailingConstraint.constant += (self.globDrawerView?.drawerModel!.miniDrawerWidth)!
            }
            self.globDrawerView!.leftDrawerMiniView.clipsToBounds = true
            //self.globSidebarView?.leftSideBarMiniImageView.image = self.model.dataImage
            self.globDrawerView!.isHidden = false
        } else if model.screenXcoordinate == .left {

            self.globDrawerView?.rightDrawerMiniView.layer.zPosition = -1
            self.globDrawerView?.rightDrawerWidthConstraint.constant = self.model.miniDrawerWidthForCircle
            self.globDrawerView?.rightDrawerMiniView.layer.cornerRadius = self.model.miniDrawerWidthForCircle / 2
            self.globDrawerView?.rightDrawerTrailingConstraint.constant = -((self.globDrawerView?.rightDrawerWidthConstraint.constant)! / 2)
            self.globDrawerView?.rightDrawerTitleLabelCenterXConstraint.constant =  -(self.model.xCoordPaddingConstant)
            self.globDrawerView?.rightDrawerContentImageCenterXConstraint.constant = -2
    
            if self.model.titleString.count > 0 {
                self.globDrawerView?.rightDrawerMiniContentImageView.isHidden = true
                self.globDrawerView?.rightDrawerMiniContentImageTopConstraint.constant *= 1.6
                self.globDrawerView?.rightDrawerMiniContentImageBottomConstraint.constant *= 3
                self.globDrawerView?.rightDrawerMiniContentImageTrailingConstraint.constant = 20
                self.globDrawerView?.rightDrawerMiniContentImageLeadingConstraint.constant += (self.globDrawerView?.drawerModel!.miniDrawerWidth)!
            } else {
                self.globDrawerView?.rightDrawerMiniContentImageTopConstraint.constant *=  3
                self.globDrawerView?.rightDrawerMiniContentImageBottomConstraint.constant *= 3
                self.globDrawerView?.rightDrawerMiniContentImageTrailingConstraint.constant = 20
                self.globDrawerView?.rightDrawerMiniContentImageLeadingConstraint.constant += (self.globDrawerView?.drawerModel!.miniDrawerWidth)!
            }
            self.globDrawerView!.rightDrawerMiniView.clipsToBounds = true
            //self.globSidebarView?.rightSideBarMiniImageView.image = self.model.dataImage
            self.globDrawerView!.isHidden = false
        }
    }
    
    func configureStandartView(drawer:drawerView) {
        
        //ekranın sağında mı solunda mı
        if self.model.screenXcoordinate == .right {
            globDrawerView?.rightDrawerMiniWidthConstraint.constant = 0
            globDrawerView?.rightDrawerMiniView.isHidden = true
        } else if self.model.screenXcoordinate == .left {
            globDrawerView?.leftDrawerMiniWidthConstraint.constant = 0
            globDrawerView?.leftDrawerMiniView.isHidden = true
        }
        
        //label tipi
        if self.model.labelType == .downToUp  && self.model.screenXcoordinate == .right {
            globDrawerView!.leftTitleLabel.transform = CGAffineTransform(rotationAngle: -CGFloat.pi / 2)
        } else if self.model.labelType == .upToDown  && self.model.screenXcoordinate == .right {
            globDrawerView!.leftTitleLabel.transform = CGAffineTransform(rotationAngle: CGFloat.pi / 2)
        } else if self.model.labelType == .downToUp  && self.model.screenXcoordinate == .left {
            globDrawerView!.rightTitleLabel.transform = CGAffineTransform(rotationAngle: -CGFloat.pi / 2)
        } else if self.model.labelType == .upToDown  && self.model.screenXcoordinate == .left {
            globDrawerView!.rightTitleLabel.transform = CGAffineTransform(rotationAngle: CGFloat.pi / 2)
        }
        
        if self.model.screenXcoordinate == .right {
            if model.titleString.count > titleLenght {
                while model.titleString.count != titleLenght {
                    model.titleString.removeLast()
                }
            }
            globDrawerView!.leftTitleLabel.text = model.titleString
            //modele göre diğer elementlerin assign edilmesi gerek left mini viewa
        } else if self.model.screenXcoordinate == .left {
            if model.titleString.count > titleLenght {
                while model.titleString.count != titleLenght {
                    model.titleString.removeLast()
                }
            }
            globDrawerView!.rightTitleLabel.text = model.titleString
            //modele göre diğer elementlerin assign edilmesi gerek right mini viewa
        }
        
        if !self.model.isCircle {
            globDrawerView?.rightDrawerMiniContentImageView.isHidden = true
            globDrawerView?.leftDrawerMiniContentImageView.isHidden = true
        }
        
        if self.model.screenXcoordinate == .right && !(self.model.isCircle ) {
            self.globDrawerView?.leftDrawerMiniContentImageTopConstraint.constant *= 1.2
        } else if self.model.screenXcoordinate == .left && !(self.model.isCircle ) {
            self.globDrawerView?.rightDrawerMiniContentImageTopConstraint.constant *= 1.2
        }

    }
    
    @objc func imageClicked(_ sender: UITapGestureRecognizer? = nil) {
        
        shouldDismissed = true
        if let url = URL(string: self.model.linkToGo ?? "") {
            delegate?.notificationShouldDismiss(controller: self, callToActionURL: url, shouldTrack: false, additionalTrackingProperties: nil)
        }
    }

    @objc func viewClicked(_ sender: UITapGestureRecognizer? = nil) {
        
        UIView.animate(withDuration: 0.5, animations: { [self] in
            if drawerOpen {
                self.window?.layer.position = drawerFirstPosition!
                if model.screenXcoordinate == .right {
                    globDrawerView?.leftDrawerMiniArrow.text = "<"
                } else if model.screenXcoordinate == .left {
                    globDrawerView?.rightDrawerMiniArrow.text = ">"
                }
            } else {
                if model.screenXcoordinate == .right {
                    if let winPos = self.window?.layer.position {
                        self.window?.layer.position = CGPoint(x: winPos.x - globDrawerView!.width + self.model.miniDrawerWidth  , y: winPos.y)
                    }
                    globDrawerView?.leftDrawerMiniArrow.text = ">"
                } else if model.screenXcoordinate == .left {
                    if let winPos = self.window?.layer.position {
                        self.window?.layer.position = CGPoint(x: winPos.x + globDrawerView!.width - self.model.miniDrawerWidth  , y: winPos.y)
                    }
                    globDrawerView?.rightDrawerMiniArrow.text = "<"
                }
            }
        })
        drawerOpen = !drawerOpen
    }
    
    
    override func show(animated: Bool) {
        guard let sharedUIApplication = RDInstance.sharedUIApplication() else {
            return
        }
        var bounds: CGRect
        if #available(iOS 13.0, *) {
            let windowScene = sharedUIApplication
                           .connectedScenes
                           .filter { $0.activationState == .foregroundActive }
                           .first
            guard let scene = windowScene as? UIWindowScene else { return }
            bounds = scene.coordinateSpace.bounds
        } else {
            bounds = UIScreen.main.bounds
        }
        let DrawerframeWidth = bounds.size.width / 2
        
        var frameY = Double()

        if self.model.screenYcoordinate == .top {
            frameY = bounds.size.height/2 - self.model.drawerHeight * 1.5
        } else if self.model.screenYcoordinate == .bottom {
            frameY = bounds.size.height - self.model.drawerHeight * 1.5
        } else if self.model.screenYcoordinate == .middle {
            frameY = bounds.size.height / 2 - self.model.drawerHeight / 2
        }

        var frame = CGRect()
        
        if self.model.screenXcoordinate == .right {
            frame = CGRect(origin: CGPoint(x: bounds.maxX-self.model.miniDrawerWidth, y: CGFloat(frameY)), size: CGSize(width: DrawerframeWidth, height: CGFloat(self.model.drawerHeight)))
        } else if self.model.screenXcoordinate == .left {
            frame = CGRect(origin: CGPoint(x: bounds.minX-DrawerframeWidth+model.miniDrawerWidth, y: CGFloat(frameY)), size: CGSize(width: DrawerframeWidth, height: CGFloat(self.model.drawerHeight)))
        }
        
        if #available(iOS 13.0, *) {
            let windowScene = sharedUIApplication
                .connectedScenes
                .filter { $0.activationState == .foregroundActive }
                .first
            if let windowScene = windowScene as? UIWindowScene {
                window = UIWindow(frame: frame)
                window?.windowScene = windowScene
            }
        } else {
            window = UIWindow(frame: frame)
        }
        if let window = window {
            window.windowLevel = UIWindow.Level.alert
            window.clipsToBounds = false // true
            window.rootViewController = self
            window.isHidden = false
        }
        self.position = self.window?.layer.position
        configureStandartView(drawer: view as! drawerView)
    }
    
    
    fileprivate func setWindowAndAddAnimation(_ animated: Bool) {
        if let window = window {
            window.windowLevel = UIWindow.Level.alert
            window.clipsToBounds = true
            window.rootViewController = self
            window.isHidden = false
        }

        let duration = animated ? 0.1 : 0
        UIView.animate(withDuration: duration, animations: {
            self.window?.frame.origin.y -= 0
        }, completion: { _ in
            self.position = self.window?.layer.position
        })
    }
    
    override func hide(animated: Bool, completion: @escaping () -> Void) {
        
        if shouldDismissed {
            self.window?.isHidden = true
            self.window?.removeFromSuperview()
            self.window = nil
            completion()
        }
    }

    
    func addTapGestureToDrawerMiniView() {
        let tap = UITapGestureRecognizer(target: self, action: #selector(self.viewClicked(_:)))
        if self.model.screenXcoordinate == .right {
            globDrawerView?.leftDrawerMiniView.addGestureRecognizer(tap)
        } else if self.model.screenXcoordinate == .left {
            globDrawerView?.rightDrawerMiniView.addGestureRecognizer(tap)
        }
    }
    
    
    func addTapGestureToImageOfGranDrawer() {
        globDrawerView?.drawerGrandContentImageView.isUserInteractionEnabled = true
        let tap = UITapGestureRecognizer(target: self, action: #selector(self.imageClicked(_:)))
        globDrawerView?.drawerGrandContentImageView.addGestureRecognizer(tap)
        
    }
}



